/*
 * Project name:
     Virtual COM Port Demo
 * Description
     Example showing usage USB CDC device that functions as a generic virtual 
     COM port. The example echoes data sent via USART terminal.
 * Test configuration:
     MCU:             P32MX795F512L
     dev.board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
     Notes:           Example uses CDC Driver included in Windows. An .inf file
                      is supplied with the example in order to match the driver
                      with this device (user must guide driver installation 
                      wizard to the VCPDriver folder).

                      Turn on USB VBUS switch.
 */

#include <stdint.h>

// Buffer of 64 bytes
char buffer[64];

void USB1Interrupt() iv IVT_USB_1 ilevel 7 ics ICS_SRS{
  // Call library interrupt handler routine
  USBDev_IntHandler();
}

void UART1_Isr() iv IVT_UART_1 ilevel 6 ics ICS_SOFT {
     int j;
     
     j = 0;

     while (UART1_Data_Ready()) {
           *(buffer+j++) = UART1_Read();
     }

     USBDev_CDCSendData(buffer, j);
     
     U1RXIF_bit = 0;
}

void USBDev_CDCParamsChanged(){
}

extern const uint8_t _USB_CDC_BULK_EP_IN;   // Data interface IN endpoint
extern const uint8_t _USB_CDC_BULK_EP_OUT;  // Data interface OUT endpoint

void USBDev_CDCDataReceived(uint16_t size){
  int i;
  
  i = 0;
  // Prepare receive buffer
  USBDev_CDCSetReceiveBuffer(buffer);
  // Send back received packet
  for (i=0; i<size; i++) UART1_Write(*(buffer+i));
  //USBDev_CDCSendData(buffer, size);
}

void main() {
  trisd.f11 = 0;

  // Reset Parani ESD 110
  latd.f11 = 0;
  Delay_ms(1500);
  latd.f11 = 1;
  
  UART1_Init(9600);               // Initialize UART module at 9600 bps
  Delay_ms(100);                  // Wait for UART module to stabilize
  
  AD1PCFG = 0xFFFF;                        // Configure AN pins as digital I/O

  // Initialize CDC Class
  USBDev_CDCInit();
  
  // Initialize USB device module
  USBDev_Init();
 
  // Enable USB device interrupt
  USBIE_bit = 1;
  IPC11bits.USBIP = 7;
  U1IP0_bit = 0;
  U1IP1_bit = 1;
  U1IP2_bit = 1;
  U1RXIE_bit = 1;
  EnableInterrupts();

  // Wait until device is configured (enumeration is successfully finished)
  while(USBDev_GetDeviceState() != _USB_DEV_STATE_CONFIGURED)
    ;
 
  // Set receive buffer where received data is stored
  USBDev_CDCSetReceiveBuffer(buffer);

  // Infinite loop
  while(1){
 
  }

}